import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_group_member_full_info.dart';
import 'package:tim_ui_kit/business_logic/view_models/tui_group_profile_view_model.dart';
import 'package:tim_ui_kit/business_logic/view_models/tui_theme_view_model.dart';
import 'package:tim_ui_kit/data_services/services_locatar.dart';
import 'package:tim_ui_kit/ui/utils/color.dart';
import 'package:tim_ui_kit/ui/widgets/group_member_list.dart';

class AtText extends StatefulWidget {
  final String? groupID;
  // some Group type cant @all
  final String? groupType;
  const AtText({
    this.groupID,
    this.groupType,
    Key? key,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _AtTextState();
}

class _AtTextState extends State<AtText> {
  final TUIGroupProfileViewModel _model = TUIGroupProfileViewModel();
  @override
  void initState() {
    super.initState();
    if (widget.groupID != null) {
      _model.loadData(widget.groupID!);
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  _onTapMemberItem(V2TimGroupMemberFullInfo memberInfo) {
    Navigator.pop(context, memberInfo);
  }

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        ChangeNotifierProvider.value(value: serviceLocator<TUIThemeViewModel>())
      ],
      builder: (context, w) {
        return Consumer<TUIThemeViewModel>(
            builder: (context, themeModel, child) {
          return Scaffold(
              appBar: AppBar(
                shadowColor: themeModel.theme.weakBackgroundColor,
                iconTheme: const IconThemeData(
                  color: Colors.white,
                ),
                flexibleSpace: Container(
                  decoration: BoxDecoration(
                    gradient: LinearGradient(colors: [
                      themeModel.theme.lightPrimaryColor ??
                          CommonColor.lightPrimaryColor,
                      themeModel.theme.primaryColor ?? CommonColor.primaryColor
                    ]),
                  ),
                ),
                leading: Row(
                  children: [
                    IconButton(
                      padding: const EdgeInsets.only(left: 16),
                      constraints: const BoxConstraints(),
                      icon: Image.asset(
                        'images/arrow_back.png',
                        package: 'tim_ui_kit',
                        height: 34,
                        width: 34,
                      ),
                      onPressed: () async {
                        Navigator.pop(context);
                      },
                    ),
                  ],
                ),
                centerTitle: true,
                leadingWidth: 100,
                title: const Text(
                  "选择提醒人",
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 17,
                  ),
                ),
              ),
              body: ChangeNotifierProvider.value(
                  value: _model,
                  child: Consumer<TUIGroupProfileViewModel>(
                      builder: ((context, value, child) {
                    return GroupProfileMemberList(
                      groupType: widget.groupType ?? "",
                      memberList: value.groupMemberList ?? [],
                      onTapMemberItem: _onTapMemberItem,
                      canAtAll: true,
                      canSlideDelete: false,
                      canSearchMember: true,
                    );
                  }))));
        });
      },
    );
  }
}
