import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:provider/provider.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_group_member_full_info.dart';
import 'package:tim_ui_kit/business_logic/view_models/tui_group_profile_view_model.dart';
import 'package:tim_ui_kit/business_logic/view_models/tui_theme_view_model.dart';
import 'package:tim_ui_kit/data_services/services_locatar.dart';
import 'package:tim_ui_kit/i18n/i18n_utils.dart';
import 'package:tim_ui_kit/tim_ui_kit.dart';
import 'package:tim_ui_kit/ui/utils/color.dart';
import 'package:tim_ui_kit/ui/widgets/group_member_list.dart';

class GroupProfileMemberListPage extends StatelessWidget {
  final List<V2TimGroupMemberFullInfo?> memberList;
  final TUIGroupProfileViewModel model;

  const GroupProfileMemberListPage({
    Key? key,
    required this.memberList,
    required this.model,
  }) : super(key: key);

  _kickedOffMember(String userID) async {
    final response = await model.kickOffMember([userID]);
    if (response.code != 0) {
      Fluttertoast.showToast(
        msg: response.desc,
        gravity: ToastGravity.BOTTOM,
        timeInSecForIosWeb: 1,
        textColor: Colors.white,
        backgroundColor: Colors.black,
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider.value(
        value: serviceLocator<TUIThemeViewModel>(),
        child: Consumer<TUIThemeViewModel>(builder: (context, value, child) {
          final theme = value.theme;
          String groupMemberNum = memberList.length.toString();
          final I18nUtils ttBuild = I18nUtils(context);
          return Scaffold(
              appBar: AppBar(
                  title: Text(
                    ttBuild.imt_para("群成员({{groupMemberNum}}人)",
                            "群成员($groupMemberNum人)")(
                        groupMemberNum: groupMemberNum),
                    style: const TextStyle(color: Colors.white, fontSize: 17),
                  ),
                  shadowColor: theme.weakBackgroundColor,
                  flexibleSpace: Container(
                    decoration: BoxDecoration(
                      gradient: LinearGradient(colors: [
                        theme.lightPrimaryColor ??
                            CommonColor.lightPrimaryColor,
                        theme.primaryColor ?? CommonColor.primaryColor
                      ]),
                    ),
                  ),
                  iconTheme: const IconThemeData(
                    color: Colors.white,
                  )),
              body: ChangeNotifierProvider.value(
                  value: model,
                  child: Consumer<TUIGroupProfileViewModel>(
                      builder: ((context, value, child) {
                    return GroupProfileMemberList(
                        canSearchMember: true,
                        memberList: value.groupMemberList ?? [],
                        removeMember: _kickedOffMember);
                  }))));
        }));
  }
}

class DeleteGroupMemberPage extends StatefulWidget {
  final TUIGroupProfileViewModel model;

  const DeleteGroupMemberPage({Key? key, required this.model})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _DeleteGroupMemberPageState();
}

class _DeleteGroupMemberPageState extends State<DeleteGroupMemberPage> {
  List<V2TimGroupMemberFullInfo> selectedGroupMember = [];

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider.value(
        value: serviceLocator<TUIThemeViewModel>(),
        child: Consumer<TUIThemeViewModel>(builder: (context, value, child) {
          final theme = value.theme;
          final I18nUtils ttBuild = I18nUtils(context);
          return Scaffold(
              appBar: AppBar(
                  title: Text(
                    ttBuild.imt("删除群成员"),
                    style: const TextStyle(color: Colors.white, fontSize: 17),
                  ),
                  actions: [
                    TextButton(
                      onPressed: () async {
                        if (selectedGroupMember.isNotEmpty) {
                          final userIDs =
                              selectedGroupMember.map((e) => e.userID).toList();
                          final res = await widget.model.kickOffMember(userIDs);
                          if (res.code != 0) {
                            Fluttertoast.showToast(
                              msg: res.desc,
                              gravity: ToastGravity.BOTTOM,
                              timeInSecForIosWeb: 1,
                              textColor: Colors.white,
                              backgroundColor: Colors.black,
                            );
                          }
                          Navigator.pop(context);
                        }
                      },
                      child: Text(
                        ttBuild.imt("确定"),
                        style: const TextStyle(
                          color: Colors.white,
                          fontSize: 16,
                        ),
                      ),
                    )
                  ],
                  shadowColor: theme.weakBackgroundColor,
                  flexibleSpace: Container(
                    decoration: BoxDecoration(
                      gradient: LinearGradient(colors: [
                        theme.lightPrimaryColor ??
                            CommonColor.lightPrimaryColor,
                        theme.primaryColor ?? CommonColor.primaryColor
                      ]),
                    ),
                  ),
                  iconTheme: const IconThemeData(
                    color: Colors.white,
                  )),
              body: ChangeNotifierProvider.value(
                  value: widget.model,
                  child: Consumer<TUIGroupProfileViewModel>(
                      builder: ((context, value, child) {
                    return GroupProfileMemberList(
                      memberList: value.groupMemberList
                              ?.where((value) =>
                                  value?.role ==
                                  GroupMemberRoleType
                                      .V2TIM_GROUP_MEMBER_ROLE_MEMBER)
                              .toList() ??
                          [],
                      canSelectMember: true,
                      canSlideDelete: false,
                      canSearchMember: true,
                      onSelectedMemberChange: (selectedMember) {
                        selectedGroupMember = selectedMember;
                      },
                    );
                  }))));
        }));
  }
}

class AddGroupMemberPage extends StatefulWidget {
  final TUIGroupProfileViewModel model;

  const AddGroupMemberPage({Key? key, required this.model}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _AddGroupMemberPageState();
}

class _AddGroupMemberPageState extends State<AddGroupMemberPage> {
  List<V2TimFriendInfo> selectedContacter = [];

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider.value(
        value: serviceLocator<TUIThemeViewModel>(),
        child: Consumer<TUIThemeViewModel>(builder: (context, value, child) {
          final theme = value.theme;
          final I18nUtils ttBuild = I18nUtils(context);
          return Scaffold(
              appBar: AppBar(
                  title: Text(
                    ttBuild.imt("添加群成员"),
                    style: const TextStyle(color: Colors.white, fontSize: 17),
                  ),
                  actions: [
                    TextButton(
                      onPressed: () async {
                        if (selectedContacter.isNotEmpty) {
                          final userIDs =
                              selectedContacter.map((e) => e.userID).toList();
                          final res =
                              await widget.model.inviteUserToGroup(userIDs);
                          if (res.code != 0) {
                            Fluttertoast.showToast(
                              msg: res.desc,
                              gravity: ToastGravity.BOTTOM,
                              timeInSecForIosWeb: 1,
                              textColor: Colors.white,
                              backgroundColor: Colors.black,
                            );
                          }
                          Navigator.pop(context);
                        }
                      },
                      child: Text(
                        ttBuild.imt("确定"),
                        style: const TextStyle(
                          color: Colors.white,
                          fontSize: 16,
                        ),
                      ),
                    )
                  ],
                  shadowColor: theme.weakDividerColor,
                  flexibleSpace: Container(
                    decoration: BoxDecoration(
                      gradient: LinearGradient(colors: [
                        theme.lightPrimaryColor ??
                            CommonColor.lightPrimaryColor,
                        theme.primaryColor ?? CommonColor.primaryColor
                      ]),
                    ),
                  ),
                  iconTheme: const IconThemeData(
                    color: Colors.white,
                  )),
              body: ChangeNotifierProvider.value(
                  value: widget.model,
                  child: Consumer<TUIGroupProfileViewModel>(
                      builder: ((context, value, child) {
                    if (value.contactList != null) {
                      return ContactList(
                        contactList: value.contactList!,
                        isCanSelectMemberItem: true,
                        onSelectedMemberItemChange: (selectedMember) {
                          selectedContacter = selectedMember;
                        },
                      );
                    }
                    return Container();
                  }))));
        }));
  }
}
